@@ -33,6 +33,7 @@
 #include <unique/uniqueapp.h>
 
 #include <bluetooth-client.h>
+#include <bluetooth-client-private.h>
 #include <bluetooth-chooser.h>
 #include <bluetooth-killswitch.h>
 
@@ -105,7 +106,7 @@ void browse_callback(GObject *widget, gpointer user_data)
 
 	address = g_strdup (g_object_get_data (widget, "address"));
 	if (address == NULL) {
-		GtkWidget *dialog, *selector, *button, *image;
+		GtkWidget *dialog, *selector, *button;
 		int response_id;
 
 		dialog = gtk_dialog_new_with_buttons(_("Select Device to Browse"), NULL,
@@ -113,8 +114,6 @@ void browse_callback(GObject *widget, gpointer user_data)
 						     GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT,
 						     NULL);
 		button = gtk_dialog_add_button (GTK_DIALOG (dialog), _("_Browse"), GTK_RESPONSE_ACCEPT);
-		image = gtk_image_new_from_icon_name (GTK_STOCK_CONNECT, GTK_ICON_SIZE_BUTTON);
-		gtk_button_set_image (GTK_BUTTON (button), image);
 		gtk_dialog_set_response_sensitive(GTK_DIALOG(dialog),
 						  GTK_RESPONSE_ACCEPT, FALSE);
 		gtk_window_set_default_size(GTK_WINDOW(dialog), 480, 400);
@@ -126,14 +125,15 @@ void browse_callback(GObject *widget, gpointer user_data)
 		gtk_container_set_border_width(GTK_CONTAINER(selector), 5);
 		gtk_widget_show(selector);
 		g_object_set(selector,
-			     "show-search", TRUE,
-			     "show-device-category", TRUE,
+			     "show-searching", FALSE,
+			     "show-device-category", FALSE,
 			     "show-device-type", TRUE,
+			     "device-category-filter", BLUETOOTH_CATEGORY_PAIRED_OR_TRUSTED,
+			     "device-service-filter", "OBEXFileTransfer",
 			     NULL);
 		g_signal_connect(selector, "selected-device-changed",
 				 G_CALLBACK(select_device_changed), dialog);
 		gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), selector);
-		bluetooth_chooser_start_discovery (BLUETOOTH_CHOOSER (selector));
 
 		address = NULL;
 		response_id = gtk_dialog_run (GTK_DIALOG (dialog));
@@ -739,30 +739,34 @@ update_device_list (GtkTreeIter *parent)
 			g_object_set_data_full (G_OBJECT (action),
 						"merge-id", GUINT_TO_POINTER (menu_merge_id), NULL);
 
-			if (services != NULL) {
-				/* The status menu item */
-				status = add_menu_item (address,
-							"status",
-							is_connected ? _("Connected") : _("Disconnected"),
-							uimanager,
-							menu_merge_id,
-							NULL);
-				gtk_action_set_sensitive (status, FALSE);
+			/* The status menu item */
+			status = add_menu_item (address,
+						"status",
+						is_connected ? _("Connected") : _("Disconnected"),
+						uimanager,
+						menu_merge_id,
+						NULL);
+			gtk_action_set_sensitive (status, FALSE);
 
+			if (services != NULL) {
 				action_path = g_strdup_printf ("/bluetooth-applet-popup/devices-placeholder/%s/%s-status",
 							       address, address);
 				action_set_bold (uimanager, status, action_path);
 				g_free (action_path);
-
-				/* The connect button */
-				oper = add_menu_item (address,
-						      "action",
-						      is_connected ? _("Disconnect") : _("Connect"),
-						      uimanager,
-						      menu_merge_id,
-						      G_CALLBACK (on_connect_activate));
+			} else {
+				gtk_action_set_visible (status, FALSE);
 			}
 
+			/* The connect button */
+			oper = add_menu_item (address,
+					      "action",
+					      is_connected ? _("Disconnect") : _("Connect"),
+					      uimanager,
+					      menu_merge_id,
+					      G_CALLBACK (on_connect_activate));
+			if (services == NULL)
+				gtk_action_set_visible (oper, FALSE);
+
 			add_separator_item (address, "connect-sep", uimanager, menu_merge_id);
 
 			/* The Send to... button */
@@ -805,10 +809,13 @@ update_device_list (GtkTreeIter *parent)
 			}
 		} else {
 			gtk_action_set_label (action, name);
-			if (status != NULL)
+
+			gtk_action_set_visible (status, services != NULL);
+			gtk_action_set_visible (oper, services != NULL);
+			if (services != NULL) {
 				set_device_status_label (address, is_connected ? CONNECTED : DISCONNECTED);
-			if (oper != NULL)
 				gtk_action_set_label (oper, is_connected ? _("Disconnect") : _("Connect"));
+			}
 		}
 		g_free (name);
 
