@@ -31,6 +31,7 @@
 #include <glib/gi18n-lib.h>
 
 #include "bluetooth-client.h"
+#include "bluetooth-client-private.h"
 #include "bluetooth-chooser.h"
 #include "gnome-bluetooth-enum-types.h"
 #include "bling-spinner.h"
@@ -186,10 +187,10 @@ set_search_label (BluetoothChooser *self, gboolean state)
 
 /**
  * bluetooth_chooser_start_discovery:
- * @self: a #BluetoothChooser widget
+ * @self: a #BluetoothChooser widget.
  *
  * Starts a discovery on the default Bluetooth adapter. Note that this will
- * only work if the Search button is visible, as otherwise the user has no
+ * only work if the Search label is visible, as otherwise the user has no
  * visual feedback that the process is on-going.
  *
  * See also: #BluetoothChooser:show-searching
@@ -208,7 +209,7 @@ bluetooth_chooser_start_discovery (BluetoothChooser *self)
 
 /**
  * bluetooth_chooser_stop_discovery:
- * @self: a #BluetoothChooser widget
+ * @self: a #BluetoothChooser widget.
  *
  * Stops a discovery started with #bluetooth_chooser_start_discovery.
  */
@@ -241,9 +242,10 @@ bluetooth_chooser_get_selected_device_data (BluetoothChooser *self, guint column
 
 /**
  * bluetooth_chooser_get_selected_device:
- * @self: a #BluetoothChooser widget
+ * @self: a #BluetoothChooser widget.
+ *
+ * Return value: the Bluetooth address for the currently selected device, or %NULL.
  *
- * Return value: the Bluetooth address for the currently selected device, or %NULL
  */
 gchar *
 bluetooth_chooser_get_selected_device (BluetoothChooser *self)
@@ -253,9 +255,10 @@ bluetooth_chooser_get_selected_device (BluetoothChooser *self)
 
 /**
  * bluetooth_chooser_get_selected_device_name:
- * @self: a #BluetoothChooser widget
+ * @self: a #BluetoothChooser widget.
+ *
+ * Return value: the name for the currently selected device, or %NULL.
  *
- * Return value: the name for the currently selected device, or %NULL
  */
 gchar *
 bluetooth_chooser_get_selected_device_name (BluetoothChooser *self)
@@ -265,9 +268,10 @@ bluetooth_chooser_get_selected_device_name (BluetoothChooser *self)
 
 /**
  * bluetooth_chooser_get_selected_device_icon:
- * @self: a #BluetoothChooser widget
+ * @self: a #BluetoothChooser widget.
+ *
+ * Return value: the icon name to use to represent the currently selected device, or %NULL.
  *
- * Return value: the icon name to use to represent the currently selected device, or %NULL
  */
 gchar *
 bluetooth_chooser_get_selected_device_icon (BluetoothChooser *self)
@@ -277,9 +281,10 @@ bluetooth_chooser_get_selected_device_icon (BluetoothChooser *self)
 
 /**
  * bluetooth_chooser_get_selected_device_type:
- * @self: a #BluetoothChooser widget
+ * @self: a #BluetoothChooser widget.
+ *
+ * Return value: the #BluetoothType of the device selected, or '0' if unknown.
  *
- * Return value: the type of the device selected, or '0' if unknown
  */
 BluetoothType
 bluetooth_chooser_get_selected_device_type (BluetoothChooser *self)
@@ -299,9 +304,11 @@ bluetooth_chooser_get_selected_device_type (BluetoothChooser *self)
 
 /**
  * bluetooth_chooser_get_selected_device_is_connected:
- * @self: a #BluetoothChooser widget
+ * @self: a #BluetoothChooser widget.
+ *
+ * Return value: whether the selected device is conncted to this computer,
+ * will always be %FALSE if no devices are selected.
  *
- * Return value: whether the selected device is conncted to this computer, will be %FALSE if no devices are selected
  */
 gboolean
 bluetooth_chooser_get_selected_device_is_connected (BluetoothChooser *self)
@@ -325,6 +332,7 @@ bluetooth_chooser_get_selected_device_is_connected (BluetoothChooser *self)
  * @value: An empty #GValue to set.
  *
  * Return value: %TRUE if the @value has been set.
+ *
  */
 gboolean
 bluetooth_chooser_get_selected_device_info (BluetoothChooser *self,
@@ -358,8 +366,9 @@ bluetooth_chooser_get_selected_device_info (BluetoothChooser *self,
 
 /**
  * bluetooth_chooser_set_title:
- * @self: a BluetoothChooser widget
- * @title: the widget header title
+ * @self: a BluetoothChooser widget.
+ * @title: the widget header title.
+ *
  */
 void
 bluetooth_chooser_set_title (BluetoothChooser  *self, const char *title)
@@ -683,6 +692,7 @@ create_treeview (BluetoothChooser *self)
 	/* The device name */
 	renderer = gtk_cell_renderer_text_new ();
 	gtk_tree_view_column_pack_start (column, renderer, TRUE);
+	g_object_set (G_OBJECT (renderer), "ellipsize", PANGO_ELLIPSIZE_END, NULL);
 	gtk_tree_view_column_set_cell_data_func (column, renderer,
 						 alias_to_label, NULL, NULL);
 
@@ -925,8 +935,7 @@ bluetooth_chooser_init(BluetoothChooser *self)
 	}
 
 	/* The services filter */
-	/* FIXME implement accessors */
-	priv->device_service_filter = NULL; //g_strdup ("OBEXObjectPush");
+	priv->device_service_filter = NULL;
 
 	/* if filters are not visible hide the vbox */
 	if (!priv->show_device_type && !priv->show_device_category)
@@ -974,7 +983,8 @@ enum {
 	PROP_SHOW_DEVICE_TYPE,
 	PROP_SHOW_DEVICE_CATEGORY,
 	PROP_DEVICE_TYPE_FILTER,
-	PROP_DEVICE_CATEGORY_FILTER
+	PROP_DEVICE_CATEGORY_FILTER,
+	PROP_DEVICE_SERVICE_FILTER
 };
 
 static void
@@ -1024,9 +1034,15 @@ bluetooth_chooser_set_property (GObject *object, guint prop_id,
 		set_combobox_from_filter (BLUETOOTH_CHOOSER (object));
 		break;
 	case PROP_DEVICE_CATEGORY_FILTER:
-		priv->device_category_filter = g_value_get_int (value);
+		priv->device_category_filter = g_value_get_enum (value);
 		gtk_combo_box_set_active (GTK_COMBO_BOX(priv->device_category), priv->device_category_filter);
 		break;
+	case PROP_DEVICE_SERVICE_FILTER:
+		g_free (priv->device_service_filter);
+		priv->device_service_filter = g_value_dup_string (value);
+		if (priv->filter)
+			gtk_tree_model_filter_refilter (GTK_TREE_MODEL_FILTER (priv->filter));
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
 		break;
@@ -1063,7 +1079,10 @@ bluetooth_chooser_get_property (GObject *object, guint prop_id,
 		g_value_set_int (value, priv->device_type_filter);
 		break;
 	case PROP_DEVICE_CATEGORY_FILTER:
-		g_value_set_int (value, priv->device_category_filter);
+		g_value_set_enum (value, priv->device_category_filter);
+		break;
+	case PROP_DEVICE_SERVICE_FILTER:
+		g_value_set_string (value, priv->device_service_filter);
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
@@ -1089,11 +1108,12 @@ bluetooth_chooser_class_init (BluetoothChooserClass *klass)
 	/**
 	 * BluetoothChooser::selected-device-changed:
 	 *
-	 * @bluetoothchooser: a #BluetoothChooser widget
-	 * @arg1: the Bluetooth address for the currently selected device, or %NULL
+	 * @chooser: a #BluetoothChooser widget which received the signal
+	 * @address: the Bluetooth address for the currently selected device, or %NULL
 	 *
 	 * The #BluetoothChooser:selected-device-changed signal is launched when the
-	 * selected device is changed, it will be %NULL is a device was unselected.
+	 * selected device is changed, it will be %NULL if a device was unselected.
+	 *
 	 **/
 	selection_table_signals[SELECTED_DEVICE_CHANGED] =
 		g_signal_new ("selected-device-changed",
@@ -1104,86 +1124,45 @@ bluetooth_chooser_class_init (BluetoothChooserClass *klass)
 			      g_cclosure_marshal_VOID__STRING,
 			      G_TYPE_NONE, 1, G_TYPE_STRING);
 
-	/**
-	 * BluetoothChooser:title:
-	 *
-	 * The widget header title.
-	 **/
 	g_object_class_install_property (G_OBJECT_CLASS(klass),
 					 PROP_TITLE, g_param_spec_string ("title",
-									  NULL, NULL, NULL, G_PARAM_WRITABLE));
-	/**
-	 * BluetoothChooser:device-selected:
-	 *
-	 * the Bluetooth address for the currently selected device, or %NULL
-	 **/
+									  "title", "The widget header title", NULL, G_PARAM_WRITABLE));
 	g_object_class_install_property (G_OBJECT_CLASS(klass),
 					 PROP_DEVICE_SELECTED, g_param_spec_string ("device-selected",
-										    NULL, NULL, NULL, G_PARAM_READABLE));
-	/**
-	 * BluetoothChooser:show-pairing:
-	 *
-	 * Whether to show the pairing column in the tree.
-	 **/
+										    "device-selected", "the Bluetooth address for the currently selected device, or %NULL", NULL, G_PARAM_READABLE));
 	g_object_class_install_property (G_OBJECT_CLASS(klass),
 					 PROP_SHOW_PAIRING, g_param_spec_boolean ("show-pairing",
-										  NULL, NULL, FALSE, G_PARAM_READWRITE));
-	/**
-	 * BluetoothChooser:show-connected:
-	 *
-	 * Whether to show the connected column in the tree.
-	 **/
+										  "show-pairing", "Whether to show the pairing column in the tree.", FALSE, G_PARAM_READWRITE));
 	g_object_class_install_property (G_OBJECT_CLASS(klass),
 					 PROP_SHOW_CONNECTED, g_param_spec_boolean ("show-connected",
-										    NULL, NULL, FALSE, G_PARAM_READWRITE));
-	/**
-	 * BluetoothChooser:show-searching:
-	 *
-	 * Whether to show the Searching label , this is necessary if you want to programmatically
-	 * start a discovery, using bluetooth_chooser_start_discovery()
-	 **/
+										    "show-connected", "Whether to show the connected column in the tree.", FALSE, G_PARAM_READWRITE));
 	g_object_class_install_property (G_OBJECT_CLASS(klass),
 					 PROP_SHOW_SEARCHING, g_param_spec_boolean ("show-searching",
-										 NULL, NULL, FALSE, G_PARAM_READWRITE));
-	/**
-	 * BluetoothChooser:show-device-type:
-	 *
-	 * Whether to show the device type filter
-	 **/
+										    "show-searching",
+										    "Whether to show the Searching label , this is necessary if you want to programmatically start a discovery, using bluetooth_chooser_start_discovery()",
+										    FALSE, G_PARAM_READWRITE));
 	g_object_class_install_property (G_OBJECT_CLASS(klass),
 					 PROP_SHOW_DEVICE_TYPE, g_param_spec_boolean ("show-device-type",
-										      NULL, NULL, TRUE, G_PARAM_READWRITE));
-	/**
-	 * BluetoothChooser:show-device-category:
-	 *
-	 * Whether to show the device category filter
-	 **/
+										      "show-device-type", "Whether to show the device type filter", TRUE, G_PARAM_READWRITE));
 	g_object_class_install_property (G_OBJECT_CLASS(klass),
 					 PROP_SHOW_DEVICE_CATEGORY, g_param_spec_boolean ("show-device-category",
-											  NULL, NULL, TRUE, G_PARAM_READWRITE));
-	/**
-	 * BluetoothChooser:device-type-filter:
-	 *
-	 * FIXME
-	 **/
+											  "show-device-category", "Whether to show the device category filter", TRUE, G_PARAM_READWRITE));
 	for (i = 0, max_filter_val = 0 ; i < _BLUETOOTH_TYPE_NUM_TYPES; i++)
 		max_filter_val += 1 << i;
 	g_object_class_install_property (G_OBJECT_CLASS(klass),
-					 PROP_DEVICE_TYPE_FILTER, g_param_spec_int ("device-type-filter", NULL, NULL,
-										    1, max_filter_val, 1, G_PARAM_READWRITE));
-	/**
-	 * BluetoothChooser:device-category-filter:
-	 *
-	 * FIXME
-	 **/
+					 PROP_DEVICE_TYPE_FILTER, g_param_spec_int ("device-type-filter",
+					 					    "device-type-filter", "A bitmask of #BluetoothType to show", 1, max_filter_val, 1, G_PARAM_READWRITE));
 	g_object_class_install_property (G_OBJECT_CLASS(klass),
-					 PROP_DEVICE_CATEGORY_FILTER, g_param_spec_int ("device-category-filter", NULL, NULL,
-					 						0, BLUETOOTH_CATEGORY_NUM_CATEGORIES, 0, G_PARAM_READWRITE));
+					 PROP_DEVICE_CATEGORY_FILTER, g_param_spec_enum ("device-category-filter",
+											 "device-category-filter", "The #BluetoothCategory to show", BLUETOOTH_TYPE_CATEGORY, BLUETOOTH_CATEGORY_ALL, G_PARAM_READWRITE));
+	g_object_class_install_property (G_OBJECT_CLASS(klass),
+					 PROP_DEVICE_SERVICE_FILTER, g_param_spec_string ("device-service-filter",
+											  "device-service-filter", "A string representing the service to filter for", NULL, G_PARAM_WRITABLE));
 }
 
 /**
  * bluetooth_chooser_new:
- * @title: the widget header title
+ * @title: the widget header title, if %NULL, the widget header will be hidden.
  *
  * Return value: A #BluetoothChooser widget
  **/
